/******************************************************************************************
 *
 * CHECK_DISK.C
 *
 * Program: Disk space plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-22-1999
 *
 * Command line: CHECK_DISK <wusp> <cusp> <filesystem>
 *
 * Description:
 *
 * This plugin will use the /bin/df command to check the free space on currently mounted
 * filesystems.  If the percent used disk space is above <cusp>, a STATE_CRITICAL is returned.
 * If the percent used disk space is above <wusp>, a STATE_WARNING is returned.  If the
 * speicified filesystem cannot be read, a STATE_CRITICAL is returned, other errors with
 * reading the output result in a STATE_UNKNOWN error.  
 *
 * Notes:
 *  - IRIX support added by Charlie Cook 4-16-1999
 *
 *****************************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256


int main(int argc, char **argv){
	int wusp;
	int cusp;
	int usp=0;
	int result;
	int line;
	int found;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char file_system[256];
	char total_disk[32];
	char used_disk[32];
	char free_disk[32];
	char percent_used[32];
	char mount_point[256];
	char *temp_buffer;

	if(argc!=4){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Disk usage plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 03-14-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wusp> <cusp> <filesystem>\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wusp> = Percent of used space necessary to result in a WARNING state\n");
		printf(" <cusp> = Percent of used space necessary to result in a CRITICAL state\n");
		printf("\n");
		printf("This plugin will check the percent of used disk space on a mounted\n");
		printf("file system and generate an alert if percentage is above one of the\n");
		printf("threshold values.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the used disk space percent to use as warning and critical thresholds */
	wusp=atoi(argv[1]);
	cusp=atoi(argv[2]);

	result=STATE_OK;

	fp=popen(DF_COMMAND,"r");
	if(fp==NULL){
		printf("Could not open pipe: %s\n",DF_COMMAND);
		return STATE_UNKNOWN;
	        }

	line=0;
	found=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		line++;

		/* skip the first line of the output */
		if(line==1)
			continue;

		/* get the filesystem */
		temp_buffer=strtok(input_buffer," ");
		if(temp_buffer!=NULL)
			strcpy(file_system,temp_buffer);
		else{
			result=STATE_UNKNOWN;
			break;
		        }

#ifdef DFNOHMNOPT  /* IRIX */
		/* Pull out filesystem type */
		temp_buffer=strtok(NULL," ");
#endif

		/* get total disk space */
		temp_buffer=strtok(NULL," ");
		if(temp_buffer!=NULL)
			strcpy(total_disk,temp_buffer);
		else{
			result=STATE_UNKNOWN;
			break;
		        }
		
		/* get used disk space  */
		temp_buffer=strtok(NULL," ");
		if(temp_buffer!=NULL)
			strcpy(used_disk,temp_buffer);
		else{
			result=STATE_UNKNOWN;
			break;
		        }
		
		/* get free disk space */
		temp_buffer=strtok(NULL," ");

		if(temp_buffer!=NULL)

#if DF_NEEDS_LABEL
		        sprintf(free_disk,"%s kb",temp_buffer);
#else
			strcpy(free_disk,temp_buffer);
#endif
		
		else{
			result=STATE_UNKNOWN;
			break;
		        }
		
		/* get the percent free space */
		temp_buffer=strtok(NULL," ");
		if(temp_buffer!=NULL)
			strcpy(percent_used,temp_buffer);
		else{
			result=STATE_UNKNOWN;
			break;
		        }
		
		/* get the mount point */
		temp_buffer=strtok(NULL," ");
		if(temp_buffer!=NULL)
			strcpy(mount_point,temp_buffer);
		else{
			result=STATE_UNKNOWN;
			break;
		        }

		/* if this is the file system we're looking for, break out of the loop */
		if(!strcmp(file_system,argv[3])){
			found=1;
			break;
		        }
		
	        }

	/* close the temp file */
	pclose(fp);

	/* if we couldn't find the file system, return a CRITICAL state */
	if(!found)
		result=STATE_CRITICAL;

        /* else check the percent used space against warning and critical thresholds */
	else if(result!=STATE_UNKNOWN){
		usp=atoi(percent_used);
		if(usp>=cusp)
			result=STATE_CRITICAL;
		else if(usp>=wusp)
			result=STATE_WARNING;
	        }
	
	if(result==STATE_OK)
		printf("Disk ok - %s (%d%%) free on %s\n",free_disk,100-usp,file_system);
	else if(result==STATE_UNKNOWN)
		printf("Unable to read output\n");
	else{
		if(!found)
			printf("Disk %s not mounted or nonexistant\n",argv[3]);
		else
			printf("Only %s (%d%%) free on %s\n",free_disk,100-usp,file_system);
	        }

	return result;
        }
