/******************************************************************************************
 *
 * CHECK_SNMP.C
 *
 * Program: SNMP plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 08-14-1999
 *
 * Command line: CHECK_SNMP <ip_address> [-c community] [-o object_id] [-e eval_method] \ 
 *                          [-wv warn_value] [-cv crit_value] [-l label] [-r rate]
 *
 * Description:
 *
 *
 * Dependencies:
 *
 * This plugin uses the 'snmpget' command included with the UCD-SNMP package.  If you don't
 * have the package installed you will need to download it from http://ucd-snmp.ucdavis.edu
 * before you can use this plugin.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define TRUE	1
#define FALSE	0

#define OK	0
#define ERROR	-1

#define CHECK_PRESENT			0
#define CHECK_GREATERTHAN		1
#define CHECK_LESSTHAN			2
#define CHECK_GREATERTHANOREQUAL	3
#define CHECK_LESSTHANOREQUAL		4
#define CHECK_EQUAL			5
#define CHECK_NOTEQUAL			6

#define MAX_INPUT_BUFFER	512

#define TEMP_FILE_NAME	"check_snmp.tmp"
char temp_file[MAX_INPUT_BUFFER]="";

int my_system(char *);
int process_arguments(int,char **);

char server_name[MAX_INPUT_BUFFER];
char community[MAX_INPUT_BUFFER]="public";
char oid[MAX_INPUT_BUFFER];
char label[MAX_INPUT_BUFFER]="";
char rate[MAX_INPUT_BUFFER]="";
unsigned long warning_threshold_value=0L;
unsigned long critical_threshold_value=0L;
unsigned long response_value=0L;
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int eval_method=CHECK_PRESENT;


int main(int argc, char **argv){
	char command_line[1024];
	int result;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char response[MAX_INPUT_BUFFER];
	char *temp_ptr;
	int found;

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect arguments supplied\n");
		printf("\n");
		printf("SNMP plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 08-14-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <ip_address> [-c community] [-o object_id] [-e eval_method] [-wv warn_value] [-cv crit_value] [-l label] [-r rate]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <ip_address>  = The IP address of the device you wish to query\n");
		printf(" [community]   = An optional community string used for SNMP communication\n");
		printf("	    	 (default is \"public\")\n");
		printf(" [object_id]   = The object ID whose value you wish to query\n");
		printf(" [eval_method] = How should the data received be evaluated against the warning\n");
		printf("                 and critical thresholds specified? Valid options are:\n");
		printf("               	   PR  = The plugin will only check to make sure that some kind\n");
		printf("                         of data is present in the response (used for strings).\n");
		printf("                   GT  = The plugin will return a non-OK state if the data\n");
		printf("			 received is greater than either the warning or the.\n");
		printf("			 critical threshold values.\n");
		printf("                   GTE = The plugin will return a non-OK state if the data\n");
		printf("			 received is greater than or equal to either the warning\n");
		printf("			 or the critical threshold values.\n");
		printf("                   LT  = The plugin will return a non-OK state if the data\n");
		printf("                         received is less than either the warning or the\n");
		printf("			 critical threshold values.\n");
		printf("                   LTE = The plugin will return a non-OK state if the data\n");
		printf("                         received is less than or equal to either the warning\n");
		printf("			 or the critical threshold values.\n");
		printf("                   EQ  = The plugin will return a non-OK state if the data\n");
		printf("                         received is equal to either the warning or the critical\n");
		printf("			 threshold values.\n");
		printf("                   NE  = The plugin will return a non-OK state if the data\n");
		printf("                         received is not equal to either the warning or the\n");
		printf("			 critical threshold values.\n");
		printf(" [warn_value]  = Threshold value for data to result in a WARNING status\n");
		printf(" [crit_value]  = Threshold value for data to result in a CRITICAL status\n");
		printf(" [label]       = A label that identifies the type of data in the output message.\n");
		printf("                   Examples: \"Bytes In\", \"Errors\", \"In Discards\"\n");
		printf(" [rate]        = A label that identifies the data rate in the output message.\n");
		printf(" 		   Examples: \"B/sec\", \"packets/sec\", \"%%\"\n");
		printf("\n");
		printf("Notes:\n");
		printf("- This plugin uses the 'snmpget' command included with the UCD-SNMP package.\n");
		printf("  If you don't have the package installed, you will need to download it from\n");
		printf("  http://ucd-snmp.ucdavis.edu before you can use this plugin.\n");
		printf("- All evaluation methods other than PR expect that the valid returned from the\n");
		printf("  SNMP query is an unsigned integer.\n");
 		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the path/file name of the temp file to use */
	snprintf(temp_file,sizeof(temp_file),"%s/%s",DEFAULT_TEMP_FILE_PATH,TEMP_FILE_NAME);
	temp_file[sizeof(temp_file)-1]='\x0';

	/* create the command line */
	sprintf(command_line,"%s -v 1 %s %s %s > %s 2> %s",PATH_TO_SNMPGET,server_name,community,oid,temp_file,temp_file);

	/* run the command */
	my_system(command_line);

	result=STATE_OK;

	/* open the temp file */
	fp=fopen(temp_file,"r");
	if(fp==NULL){
		remove(temp_file);
		printf("Error: Could not open temp file for reading\n");
		return STATE_UNKNOWN;
	        }

	found=0;
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		found=1;

		/* strip the newline character from the end of the input */
		if(input_buffer[strlen(input_buffer)-1]=='\n')
			input_buffer[strlen(input_buffer)-1]='\x0';

		temp_ptr=strchr(input_buffer,'=');
		if(temp_ptr==NULL){
			printf("%s problem - %s\n",(!strcmp(label,""))?"SNMP":label,input_buffer);
			result=STATE_CRITICAL;
			break;
		        }

		strcpy(response,temp_ptr+1);

		if(eval_method==CHECK_PRESENT){
			printf("%s %s - %s %s\n",(!strcmp(label,""))?"SNMP":label,(!strcmp(response,""))?"problem":"ok",response,rate);
			result=STATE_OK;
		        }

		else{

			response_value=strtoul(response,NULL,10);

			result=STATE_OK;

			if(eval_method==CHECK_GREATERTHAN){
				if(check_critical_value==TRUE && (response_value>critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value>warning_threshold_value))
					result=STATE_WARNING;
			        }
			else if(eval_method==CHECK_GREATERTHANOREQUAL){
				if(check_critical_value==TRUE && (response_value>=critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value>=warning_threshold_value))
					result=STATE_WARNING;
			        }
			else if(eval_method==CHECK_LESSTHAN){
				if(check_critical_value==TRUE && (response_value<critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value<warning_threshold_value))
					result=STATE_WARNING;
			        }
			else if(eval_method==CHECK_LESSTHANOREQUAL){
				if(check_critical_value==TRUE && (response_value<=critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value<=warning_threshold_value))
					result=STATE_WARNING;
			        }
			else if(eval_method==CHECK_EQUAL){
				if(check_critical_value==TRUE && (response_value==critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value==warning_threshold_value))
					result=STATE_WARNING;
			        }
			else if(eval_method==CHECK_NOTEQUAL){
				if(check_critical_value==TRUE && (response_value!=critical_threshold_value))
					result=STATE_CRITICAL;
				else if(check_warning_value==TRUE && (response_value!=warning_threshold_value))
					result=STATE_WARNING;
			        }
			else{
				printf("Unknown evaluation method\n");
				return STATE_UNKNOWN;
		                }

			printf("%s %s - %lu %s\n",(!strcmp(label,""))?"SNMP":label,(result==STATE_OK)?"ok":"problem",response_value,rate);
		        }

		break;
	        }

	if(!found){
		printf("%s problem - No data recieved from host\n",(!strcmp(label,""))?"SNMP":label);
		result=STATE_UNKNOWN;
	        }

	/* close the temp file */
	fclose(fp);

	/* remove the temp file */
	remove(temp_file);

	return result;
        }



int process_arguments(int argc, char **argv){
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strcpy(server_name,argv[1]);

	for(x=3;x<=argc;x++){

		if(!strcmp(argv[x-1],"-c")){
			if(x<argc){
				strcpy(community,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-o")){
			if(x<argc){
				strcpy(oid,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-l")){
			if(x<argc){
				strcpy(label,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-r")){
			if(x<argc){
				strcpy(rate,argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wv")){
			if(x<argc){
				check_warning_value=TRUE;
				warning_threshold_value=strtoul(argv[x],NULL,10);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-cv")){
			if(x<argc){
				check_critical_value=TRUE;
				critical_threshold_value=strtoul(argv[x],NULL,10);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-e")){
			if(x<argc){

				if(!strcmp(argv[x],"PR"))
					eval_method=CHECK_PRESENT;
				else if(!strcmp(argv[x],"GT"))
					eval_method=CHECK_GREATERTHAN;
				else if(!strcmp(argv[x],"LT"))
					eval_method=CHECK_LESSTHAN;
				else if(!strcmp(argv[x],"GTE"))
					eval_method=CHECK_GREATERTHANOREQUAL;
				else if(!strcmp(argv[x],"LTE"))
					eval_method=CHECK_LESSTHANOREQUAL;
				else if(!strcmp(argv[x],"EQ"))
					eval_method=CHECK_EQUAL;
				else if(!strcmp(argv[x],"NE"))
					eval_method=CHECK_NOTEQUAL;
				else
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }


int my_system(char *command){
        int pid;
	int status;
	int result;
	extern int errno;
	extern char **environ;


	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1)
	        return STATE_UNKNOWN;  

	/* execute the command in the child process */
        if (pid==0){
		char *argv[4];
		argv[0]="sh";
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;
		execve("/bin/sh",argv,environ);
		exit(STATE_UNKNOWN);
	        }

	/* wait for the command to finish */
        do{
                if (waitpid(pid, &status, 0) == -1) {
		        if (errno != EINTR)
                                return STATE_UNKNOWN;
                        } 

		/* get the exit code returned from the program */
		result=WEXITSTATUS(status);

		/* check the bounds for the return code */
		if(result<0||result>2)
			return STATE_UNKNOWN;

		return result;
           } while(1);

        return STATE_OK;
        }

